/*======================================================================================
   format_plugin_sample.c
======================================================================================
 $RCSfile: format_plugin_sample.c$
 $Revision:$
 $Date:$
 $Author:Will Huang$
 $Revision: $
======================================================================================

======================================================================================


---------------------------------------------------------------------------

  Module Description:
     Format Plugin sample.

---------------------------------------------------------------------------*/


#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include <sys/types.h>
#include <sys/stat.h>
#include <unistd.h>

#include <hsm_plugin/matrix_barcodeid.h>
#include <hsm_plugin/matrix_plugin.h>
#include <hsm_plugin/matrix_format_plugin.h>
#include <hsm_plugin/matrix_beep_led.h>

/*===================================================================
 * Plug-in properties
 * Defined by user
 *
=====================================================================*/
#define PLUGIN_NAME EasyUSPS
#define COMPANY_NAME Plug-In Developer, Inc.
#define MAJOR_VERSION	1
#define MONOR_VERSION	1
#define BUILD_NUMBER	1
#define CERTIFICATE		100000
#define CERTIFICATE_TIME	2013/09/24 09:57:00
#define PLUGIN_GUID	abcd1234
#define FILE_NAME		EasyUSPS.plugin

#define MenuID	0xF0020

static int  Licensed;
static int Initial_enable_me=1;
static int  Enable_me=1;
static int progmd=0;
// maximum barcode length is 254
// maximum length after Base64 is 342
// plus code type, ',' and <stx>, <etx>, the maxi totally length is 346
#define MAX_TATS_BC_LEN            256
#define MAX_TATS_BF_SIZE           342       // Barcode buffer size in chars for TATS 
int  Base64Encode( unsigned char *pSrc, int nSrc, unsigned char *pDst, int nDst , int CharSize);
int  GetBarcodeIds( DataEditParam* cOrig, unsigned char *c );

/************************************************************************/
// Get the USPS defined barcode identifier
/*************************************************************************/

int GetBarcodeIds( DataEditParam* cOrig, unsigned char *c )
{
	if ( cOrig == NULL) return (-1);
	if ( c == NULL) return (-1);

	// Handle all symbologies we can
	switch ( cOrig->HHPcodeID)
	{
		// Is this a Code128?
		case WA_CODELETTER_CODE128:
			printf("This is code128\r\n");
		*c = '3';  
		break;

		// Is this a UCC/EAN128?			
		case WA_CODELETTER_UCC128:
			printf("This is EAN128\r\n");
		*c = '4';  //EAN128
		break;			

		// Is this a Code39?
		case WA_CODELETTER_CODE39:
			printf("This is Code39\r\n");
		*c = '1'; 
		break;

		// Is this a I2OF5?
		case WA_CODELETTER_INTERLEAVED2OF5:
			printf("This is I2OF5\r\n");
		*c = '2'; 
		break;

		// Is this a planet?
		case 'L':
			printf("This is planet\r\n");
		*c = '8';  //planet
		break;

		// Is this a postal?
		case 'P':
			printf("This is postal\r\n");
		*c = '7'; 
		break;

		// Is this a PDF417?
		case WA_CODELETTER_PDF417:
			printf("This is PDF417\r\n");
		*c = '5'; 
		break;

		// Is this a DataMatrix?
		case WA_CODELETTER_DATAMATRIX:
			printf("This is DataMatrix\r\n");
		*c = '6'; 
		break;

		case 'M':
			printf("This is 4-state\r\n");
		*c = '9'; 
		break;			

		// Default case
		default:
			printf("This is not customer barcode\r\n");
		*c = '?'; 
		break;
	}
	// Success
	return 0;
}


/************************************************************************/
// Base64 converstion function
/*************************************************************************/
int Base64Encode( unsigned char *pSrc, int nSrc, unsigned char *pDst, int nDst , int CharSize)
{
	char szTable[] = "ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz0123456789+/";
	int  nDstReq, nPadLen, i, j, k, m;

	// Argument checks
	if ( (pSrc==NULL)||(nSrc <= 0) || (pDst==NULL)|| (nDst <= 0) )
		return (-1);

	// Calculate the padded input buffer length
	nPadLen = ( ( ( nSrc - 1 ) / 3 ) + 1 ) * 3;

	// Pad '0' to source string.
	// there should not be any memory overflow here, because pSrc is MAX_TATS_BF_SIZE(356) long,
	// and nSrc<=MAX_TATS_BC_LEN (254)
	for( m = nSrc*CharSize; m < nPadLen*CharSize; m++)
		pSrc[m]=0;

	// Calculate the required size of the destination buffer. We need 4 bytes
	// in the destination buffer for every 3 bytes in the source buffer, plus
	// one additional character for zero-termination.
	nDstReq = ( ( nPadLen / 3 ) * 4 ) + 1;

	// Is the destination buffer large enough?
	if ( nDst < nDstReq )
		return (-1);

	// Initialize destination buffer
	memset( pDst, 0, nDst );

	// Convert to Base64 string
	for ( i = 0, j = 0, k = 0; i < nPadLen*CharSize; i+=CharSize)
	{
		// Which byte are we processing?
		switch ( (i/2) % 3 )
		{
			// First byte of three
			case 0:
				k = (pSrc[i] & 0xFC) >> 2;
				pDst[j] = szTable[k];
				j+=CharSize;
				k = (pSrc[i] & 0x03) << 4;
				break;
			// Second byte of three
			case 1:
				k += (pSrc[i] & 0xF0) >> 4;
				pDst[j] = szTable[k];
				j+=CharSize;
				k  = (pSrc[i] & 0x0F) << 2;
				break;
			// Third byte of three
			case 2:
				k += (pSrc[i] & 0xC0) >> 6;
				pDst[j] = szTable[k];
				j+=CharSize;
				k  = (pSrc[i] & 0x3F);
				pDst[j] = szTable[k];
				j+=CharSize;
				break;
		}
	}

	// Add Base64 padding if necessary
	for ( i = 0; i < nPadLen - nSrc; i++ )
		pDst[(nDstReq-i-2)*CharSize] = '=';	

	for(i=1; i< nDstReq*CharSize; i+=CharSize)
		pDst[i] = 0x00;	

	// Success, return the new string length, excluding the NULL
	return (nDstReq-1);
}



int MatrixPluginDataEdit(DataEditParam *pFormatParam)
{	
	int           j;
	unsigned char cId;
	unsigned char *strBC;                     //points to the local updated barcode string buffer
	int           iNewlen;                    //updated barcode length
	unsigned char *         bcOrig;
	unsigned short*         BcOrig;
	unsigned short *        StrBC;
	unsigned char *         bcNew;

	unsigned char *buffer = NULL;
	unsigned char  *cBarcode=NULL; //updated barcode string buffer
	FILE *fp=NULL; 
	
	fp= fopen ("PluginEnable", "r");
	if(fp == NULL)
	{
		printf("MatrixPluginDataEdit: UseInitialPluginEnable\r\n");
		
	}
	else
	{

			int tmp;
			fseek(fp, 0, SEEK_SET);
			tmp=fread(&Enable_me, 1, 1, fp);
			printf("fread Enable_me=%d\r\n", Enable_me);
			fclose(fp);
			Enable_me&=Initial_enable_me;
			if(Enable_me==0)
			{
				printf("MatrixPluginProcessingBarcode: fwrite PluginEnable,failed\r\n");
				fclose(fp);
				return -1;
			}
			else
			{
				printf("fwrite PluginlEnable %d byte\r\n", tmp);			
				fclose(fp);
				
			}
		}
	 

	if ( pFormatParam->length > MAX_TATS_BC_LEN) //maxi barcode length allowed = 256
	{
		printf("too long \n");
		return (-1);
	}

	buffer = malloc((pFormatParam->length + 100 )*(pFormatParam->CharSize));
	cBarcode = malloc(MAX_TATS_BF_SIZE);
	if(!buffer)
	return -1;

	memset(buffer, 0, (pFormatParam->length + 100 )*(pFormatParam->CharSize));

	// Initialize variables
	bcOrig  = pFormatParam->message;
	BcOrig   = (unsigned short *)bcOrig;
	memset( cBarcode, 0, MAX_TATS_BF_SIZE );
	strBC   = cBarcode;
	StrBC  =(unsigned short *)strBC;
	iNewlen = 0;

	// Get the AIM code identifiers
	if ( GetBarcodeIds( pFormatParam, &cId ) < 0 )
	{
		printf("GetAIMCodeIds failed!\n");
		return (-1);
	}

    // Reformat barcode string		

	if (pFormatParam->HHPcodeID == WA_CODELETTER_UCC128 )
	{		
		// If this is EAN128 barcode delete FNC1			
		for ( j = 0; j < pFormatParam->length; j++ )
		{
			if(pFormatParam->CharSize==1)
			{
				if((*bcOrig!=0xF1)&&(*bcOrig!=0x1D))
				{
					*(strBC++)= *(bcOrig++);  //copy everything other than FNC1
					iNewlen++;
				}
				else
					bcOrig++;
			}
			else if (pFormatParam->CharSize==2)    //unicode
			{
				if((*BcOrig!=0x00F1)&&(*BcOrig!=0x001D))
				{			
					*(StrBC++)= *(BcOrig++);  //copy everything other than FNC1
					iNewlen++;
				}
				else
					BcOrig++;					
			}

		}
	}
	else if (pFormatParam->HHPcodeID == WA_CODELETTER_PDF417) 
	{
		printf("PDF417 base64 encode\r\n");
		iNewlen = Base64Encode((unsigned char*)bcOrig, pFormatParam->length, strBC, MAX_TATS_BF_SIZE, pFormatParam->CharSize);

		if(iNewlen<0)
		{
			printf("Base64 failed! \n");
			return (-1);
		}

	}
	else if (pFormatParam->HHPcodeID == WA_CODELETTER_DATAMATRIX)
	{			
	//Use Base64 to convert
		printf("Datamatrix base64 encode\r\n");
		iNewlen = Base64Encode((unsigned char*)bcOrig, pFormatParam->length, strBC, MAX_TATS_BF_SIZE, pFormatParam->CharSize);
		
		if(iNewlen<0)
		{
			printf( "DM Base64 failed! \n");
			return -1;
		}

	}
	else
	{
		memcpy(cBarcode, pFormatParam->message, (pFormatParam->length)*(pFormatParam->CharSize));    //copy the whole barcode over
		iNewlen = pFormatParam->length;
	}
	if(pFormatParam->CharSize==1)
	{
		sprintf( (char*)buffer, "%c,", cId);  
		bcNew += strlen((char*)buffer);  
	}
	else	 if(pFormatParam->CharSize==2)
	{
		unsigned short *sbuffer = (unsigned short*)buffer;
		*sbuffer = (unsigned short)cId;		
		*(sbuffer+1) = ',';
		bcNew = (unsigned char*)(sbuffer+2);  
	}
	memcpy(bcNew, (char*)cBarcode, (iNewlen+1)*(pFormatParam->CharSize)); //add a NULL to the end
	iNewlen +=2;
	memcpy(pFormatParam->message, buffer, iNewlen*(pFormatParam->CharSize));
	pFormatParam->length = iNewlen;
	
	printf("new %d: %s\r\n",pFormatParam->length, pFormatParam->message);
	free(cBarcode);	
	free(buffer);
	return 0;
}

/** This API is called when plug-in needs to configure itself by 
 *  scanning a specific menu barcode.
 */
int MatrixPluginProcessingBarcode(char *pMenuData, int DataLength)
{
	// Add your plugin menu processing code here and return 0 if succeed and -1 if fail, 
	// system will beeps error or success according to the return value
	printf("\r\nMatrixPluginProcessingBarcode\r\n");
	printf("Menu Data is: ");
	printf(pMenuData);
	printf("\r\n");
	
	FILE *fp=NULL; 
	if ( DataLength >= 2 && strncmp((char*)pMenuData, "00", 2) == 0 )
	{
		Enable_me = 0;
		beep_led_io(MenuDifferentBeepSeq, 6);
		progmd=1;
	}
	if ( DataLength >= 2 && strncmp((char*)pMenuData, "01", 2) == 0 )
	{
		Enable_me = 1;
		beep_led_io(MenuDifferentBeepSeq, 6);
		progmd=1;
	}

	if(progmd==1)
	{
		fp= fopen ("PluginEnable", "w");
		if(fp == NULL)
		{
			printf("fopen PluginEnable,failed\r\n");
			return -1;
		}
		else
		{
			int tmp;
			fseek(fp, 0, SEEK_SET);
			tmp=fwrite(&Enable_me, 1, 1, fp);
			if(tmp==0)
			{
				printf("MatrixPluginProcessingBarcode: fwrite PluginEnable,failed\r\n");
				fclose(fp);
				return -1;
			}
			else
			{
				printf("fwrite PluginEnable %d byte\r\n", tmp);			
				fclose(fp);
				return 0;
			}
		}

	}


	return -1;
}

/** The MatrixPlulginCheckLicense API is used to validate if 
 *  the plug-in is licensed or not. This function will be called
 *  immediately when the plug-in is loaded. Plug-in should keep
 *  the result of the license validation during its runtime.If 
 *  the plug-in is not licensed, the plug-in must return -1 when
 *  any other API of it is called.
 */
int MatrixPlulginCheckLicense(char *SN)
{
	// Add your license validation code here
	// return 0 to indicate the plug-in is licensed
	// return -1 for not licensed
	printf("\r\nMatrixPlulginCheckLicense\r\n");
	printf("Serial Number: %s\r\n", SN);
	return -1;
}


/** Honeywell Gen.6 scanner use this API to retrieve plug-in
 *  Version information. The version information will be filled
 *  in the VersionInfo structure after the API returns
 *  successfully.
 */
int MatrixPluginGetVersion(VersionInfo *pInfo)
{
	// Add your Get version code here
	// Plugin should fill in the pInfo
	// with the plugin version info
	// return 0 for success and -1 for failure
	printf("\r\nMatrixPluginGetVersion\r\n");
	return 0;
}

void MatrixPluginGPIO_Plugins()
{
	DataEditApi apis;
//	apis.RevisionNumber=3;
//	printf("\r\nMatrixPluginNewfunction\r\n");

}

/********************************************** 
   Format Plugin APIs definition 
   Fill the fields with 
   user defined APIs.
 *********************************************/
static DataEditApi apis = {
	.RevisionNumber = 1,
	.DataEdit = MatrixPluginDataEdit,
	.ProcessingBarcode = MatrixPluginProcessingBarcode,
	.CheckLicense = MatrixPlulginCheckLicense,
	.GetVersion = MatrixPluginGetVersion,
 //      .GPIO_Plugins=MatrixPluginGPIO_Plugins,
	//.Newfunction=MatrixPluginNewfunction,
};


/**********************************************
   This is the initial function of plug-in
   This routine is user defined and be used
   to initilize plug-in and register plug-in
   APIs.
   Parameter:
   		plugin ---- Plug-in information structure
   Return Value:
   		-1 indicate a error and 0 to success
 *********************************************/
int init_plugin(HONPluginRawInfo *plugin)
{
	int ret = 0;
	FILE* file = NULL;
	void *pTest = NULL;
	
	printf("/******************************/\r\n");
	printf("     Sample Format Plugin \r\n");
	printf("/******************************/\r\n");
	
	


	printf("Before malloc\r\n");
	pTest = malloc(1);
	printf("after malloc\r\n");
	
	if(!pTest)
		printf("malloc fail\r\n");
	else{
		printf("malloc ok\r\n");
		free(pTest);
	}
	
	pTest = calloc(1, 10);
	if(!pTest)
		printf("calloc fail\r\n");
	else{
		printf("calloc ok\r\n");
		free(pTest);
	}
	
	file = fopen("format_cfg", "r");
	if(file){
		int r;
		printf("close file\r\n");
		r = fclose(file);
		printf("close return is %d\r\n");
	}
		unlink("format_cfg");
#if 0		
	file = fopen("format_cfg", "r");
	if(!file){
		file = fopen("format_cfg", "w");
		if (file)
		{
			printf("Open okay\r\n");
			if(fwrite("[0&1/d0/s,/f12/f13/f14/", 1, 24, file) == 24){
				printf("Write okay\r\n");
			}
			fclose(file);
		}else{
			printf("Open failed\r\n");
		}
	}


	file = fopen("format_cfg", "r");
	if (file)
	{
		char buf[32];
		int rt = 0;
		rt = fread(buf, 1, 24, file);
		printf("Read return value is %d\r\n", rt);
		if(rt == 24){
			buf[24] = 0;
			printf("Read ok\r\n");
			printf("buf is %s", buf);
		}else
			printf("Read fail\r\n");
		fclose(file);
	}else{
		printf("Open read failed\r\n");
	}
#endif	

	
	ret = register_apis(plugin, &apis);
	if(ret){
		printf("Plugin APIs Registration Failed\r\n");
		return -1;
	}
	
	return 0;

}

/**********************************************
   This is the cleanup function of plug-in
   This routine is user defined and is used
   to cleanup plug-in when remove plug-in from
   system.
 *********************************************/
void cleanup_plugin(void)
{
	return;
}

/**********************************************
   We should declare the plug-in by using
   'DECLARE_PLUGIN' macro in order that 
   Matrix can recognize the plug-in
 *********************************************/
DECLARE_PLUGIN(init_plugin, cleanup_plugin, HON_PLUGIN_FORMAT, MenuID);

/*=================================================================================
                       File Modification History
===================================================================================
 $Log:$
===================================================================================*/


